@@ -4,14 +4,13 @@ from copy import deepcopy
 from django.core.exceptions import FieldError, MultipleObjectsReturned
 from django.db import models, transaction
 from django.db.utils import IntegrityError
-from django.test import TestCase, ignore_warnings
-from django.utils import six
-from django.utils.deprecation import RemovedInDjango20Warning
-from django.utils.translation import ugettext_lazy
+from django.test import TestCase
+from django.utils.translation import gettext_lazy
 
 from .models import (
-    Article, Category, Child, City, District, First, Parent, Record, Relation,
-    Reporter, School, Student, Third, ToFieldChild,
+    Article, Category, Child, ChildNullableParent, City, Country, District,
+    First, Parent, Record, Relation, Reporter, School, Student, Third,
+    ToFieldChild,
 )
 
 
@@ -30,9 +29,6 @@ class ManyToOneTests(TestCase):
         # Article objects have access to their related Reporter objects.
         r = self.a.reporter
         self.assertEqual(r.id, self.r.id)
-        # These are strings instead of unicode strings because that's what was used in
-        # the creation of this reporter (and we haven't refreshed the data from the
-        # database, which always returns unicode strings).
         self.assertEqual((r.first_name, self.r.last_name), ('John', 'Smith'))
 
     def test_create(self):
@@ -73,8 +69,7 @@ class ManyToOneTests(TestCase):
 
         # Adding an object of the wrong type raises TypeError.
         with transaction.atomic():
-            with six.assertRaisesRegex(self, TypeError,
-                                       "'Article' instance expected, got <Reporter.*"):
+            with self.assertRaisesMessage(TypeError, "'Article' instance expected, got <Reporter:"):
                 self.r.article_set.add(self.r2)
         self.assertQuerysetEqual(
             self.r.article_set.all(),
@@ -117,10 +112,9 @@ class ManyToOneTests(TestCase):
     def test_reverse_assignment_deprecation(self):
         msg = (
             "Direct assignment to the reverse side of a related set is "
-            "deprecated due to the implicit save() that happens. Use "
-            "article_set.set() instead."
+            "prohibited. Use article_set.set() instead."
         )
-        with self.assertRaisesMessage(RemovedInDjango20Warning, msg):
+        with self.assertRaisesMessage(TypeError, msg):
             self.r2.article_set = []
 
     def test_assign(self):
@@ -159,6 +153,34 @@ class ManyToOneTests(TestCase):
         self.assertFalse(hasattr(self.r2.article_set, 'remove'))
         self.assertFalse(hasattr(self.r2.article_set, 'clear'))
 
+    def test_assign_fk_id_value(self):
+        parent = Parent.objects.create(name='jeff')
+        child1 = Child.objects.create(name='frank', parent=parent)
+        child2 = Child.objects.create(name='randy', parent=parent)
+        parent.bestchild = child1
+        parent.save()
+        parent.bestchild_id = child2.pk
+        parent.save()
+        self.assertEqual(parent.bestchild_id, child2.pk)
+        self.assertFalse(Parent.bestchild.is_cached(parent))
+        self.assertEqual(parent.bestchild, child2)
+        self.assertTrue(Parent.bestchild.is_cached(parent))
+        # Reassigning the same value doesn't clear cached instance.
+        parent.bestchild_id = child2.pk
+        self.assertTrue(Parent.bestchild.is_cached(parent))
+
+    def test_assign_fk_id_none(self):
+        parent = Parent.objects.create(name='jeff')
+        child = Child.objects.create(name='frank', parent=parent)
+        parent.bestchild = child
+        parent.save()
+        parent.bestchild_id = None
+        parent.save()
+        self.assertIsNone(parent.bestchild_id)
+        self.assertFalse(Parent.bestchild.is_cached(parent))
+        self.assertIsNone(parent.bestchild)
+        self.assertTrue(Parent.bestchild.is_cached(parent))
+
     def test_selects(self):
         self.r.article_set.create(headline="John's second story", pub_date=datetime.date(2005, 7, 29))
         self.r2.article_set.create(headline="Paul's story", pub_date=datetime.date(2006, 1, 17))
@@ -183,7 +205,7 @@ class ManyToOneTests(TestCase):
             Article.objects.filter(reporter__first_name__exact='John'),
             ["<Article: John's second story>", "<Article: This is a test>"]
         )
-        # Check that implied __exact also works
+        # Implied __exact also works
         self.assertQuerysetEqual(
             Article.objects.filter(reporter__first_name='John'),
             ["<Article: John's second story>", "<Article: This is a test>"]
@@ -204,7 +226,7 @@ class ManyToOneTests(TestCase):
                 where=["many_to_one_reporter.last_name='Smith'"]),
             ["<Article: John's second story>", "<Article: This is a test>"]
         )
-        # ... and should work fine with the unicode that comes out of forms.Form.cleaned_data
+        # ... and should work fine with the string that comes out of forms.Form.cleaned_data
         self.assertQuerysetEqual(
             (Article.objects
                 .filter(reporter__first_name__exact='John')
@@ -314,7 +336,7 @@ class ManyToOneTests(TestCase):
         )
         self.assertQuerysetEqual(Reporter.objects.filter(article__reporter__exact=self.r).distinct(), john_smith)
 
-        # Check that implied __exact also works.
+        # Implied __exact also works.
         self.assertQuerysetEqual(Reporter.objects.filter(article__reporter=self.r).distinct(), john_smith)
 
         # It's possible to use values() calls across many-to-one relations.
@@ -327,8 +349,8 @@ class ManyToOneTests(TestCase):
         self.assertEqual([d], list(qs))
 
     def test_select_related(self):
-        # Check that Article.objects.select_related().dates() works properly when
-        # there are multiple Articles with the same date but different foreign-key
+        # Article.objects.select_related().dates() works properly when there
+        # are multiple Articles with the same date but different foreign-key
         # objects (Reporters).
         r1 = Reporter.objects.create(first_name='Mike', last_name='Royko', email='royko@suntimes.com')
         r2 = Reporter.objects.create(first_name='John', last_name='Kass', email='jkass@tribune.com')
@@ -413,7 +435,8 @@ class ManyToOneTests(TestCase):
         self.assertEqual(a3.reporter.id, self.r2.id)
 
         # Get should respect explicit foreign keys as well.
-        with self.assertRaises(MultipleObjectsReturned):
+        msg = 'get() returned more than one Article -- it returned 2!'
+        with self.assertRaisesMessage(MultipleObjectsReturned, msg):
             Article.objects.get(reporter_id=self.r.id)
         self.assertEqual(
             repr(a3),
@@ -436,11 +459,11 @@ class ManyToOneTests(TestCase):
         # Same as each other
         self.assertIs(r1.article_set.__class__, r2.article_set.__class__)
 
-    def test_create_relation_with_ugettext_lazy(self):
+    def test_create_relation_with_gettext_lazy(self):
         reporter = Reporter.objects.create(first_name='John', last_name='Smith', email='john.smith@example.com')
-        lazy = ugettext_lazy('test')
+        lazy = gettext_lazy('test')
         reporter.article_set.create(headline=lazy, pub_date=datetime.date(2011, 6, 10))
-        notlazy = six.text_type(lazy)
+        notlazy = str(lazy)
         article = reporter.article_set.get()
         self.assertEqual(article.headline, notlazy)
 
@@ -467,7 +490,7 @@ class ManyToOneTests(TestCase):
         self.assertIs(c.parent, p)
 
         # But if we kill the cache, we get a new object.
-        del c._parent_cache
+        del c._state.fields_cache['parent']
         self.assertIsNot(c.parent, p)
 
         # Assigning a new object results in that object getting cached immediately.
@@ -491,7 +514,11 @@ class ManyToOneTests(TestCase):
         setattr(c, "parent", None)
 
         # You also can't assign an object of the wrong type here
-        with self.assertRaises(ValueError):
+        msg = (
+            'Cannot assign "<First: First object (1)>": "Child.parent" must '
+            'be a "Parent" instance.'
+        )
+        with self.assertRaisesMessage(ValueError, msg):
             setattr(c, "parent", First(id=1, second=1))
 
         # You can assign None to Child.parent during object creation.
@@ -524,6 +551,23 @@ class ManyToOneTests(TestCase):
         self.assertIsNot(c.parent, p)
         self.assertEqual(c.parent, p)
 
+    def test_save_nullable_fk_after_parent(self):
+        parent = Parent()
+        child = ChildNullableParent(parent=parent)
+        parent.save()
+        child.save()
+        child.refresh_from_db()
+        self.assertEqual(child.parent, parent)
+
+    def test_save_nullable_fk_after_parent_with_to_field(self):
+        parent = Parent(name='jeff')
+        child = ToFieldChild(parent=parent)
+        parent.save()
+        child.save()
+        child.refresh_from_db()
+        self.assertEqual(child.parent, parent)
+        self.assertEqual(child.parent_id, parent.name)
+
     def test_fk_to_bigautofield(self):
         ch = City.objects.create(name='Chicago')
         District.objects.create(city=ch, name='Far South')
@@ -533,6 +577,15 @@ class ManyToOneTests(TestCase):
         District.objects.create(city=ny, name='Brooklyn')
         District.objects.create(city=ny, name='Manhattan')
 
+    def test_fk_to_smallautofield(self):
+        us = Country.objects.create(name='United States')
+        City.objects.create(country=us, name='Chicago')
+        City.objects.create(country=us, name='New York')
+
+        uk = Country.objects.create(name='United Kingdom', id=2 ** 11)
+        City.objects.create(country=uk, name='London')
+        City.objects.create(country=uk, name='Edinburgh')
+
     def test_multiple_foreignkeys(self):
         # Test of multiple ForeignKeys to the same model (bug #7125).
         c1 = Category.objects.create(name='First')
@@ -557,7 +610,8 @@ class ManyToOneTests(TestCase):
 
         p = Parent.objects.create(name="Parent")
         c = Child.objects.create(name="Child", parent=p)
-        with self.assertRaises(ValueError):
+        msg = 'Cannot assign "%r": "Child.parent" must be a "Parent" instance.' % c
+        with self.assertRaisesMessage(ValueError, msg):
             Child.objects.create(name="Grandchild", parent=c)
 
     def test_fk_instantiation_outside_model(self):
@@ -567,20 +621,19 @@ class ManyToOneTests(TestCase):
         self.assertEqual('id', cat.remote_field.get_related_field().name)
 
     def test_relation_unsaved(self):
-        # Test that the <field>_set manager does not join on Null value fields (#17541)
+        # The <field>_set manager does not join on Null value fields (#17541)
         Third.objects.create(name='Third 1')
         Third.objects.create(name='Third 2')
         th = Third(name="testing")
-        # The object isn't saved an thus the relation field is null - we won't even
+        # The object isn't saved and thus the relation field is null - we won't even
         # execute a query in this case.
         with self.assertNumQueries(0):
             self.assertEqual(th.child_set.count(), 0)
         th.save()
-        # Now the model is saved, so we will need to execute an query.
+        # Now the model is saved, so we will need to execute a query.
         with self.assertNumQueries(1):
             self.assertEqual(th.child_set.count(), 0)
 
-    @ignore_warnings(category=RemovedInDjango20Warning)  # for use_for_related_fields deprecation
     def test_related_object(self):
         public_school = School.objects.create(is_public=True)
         public_student = Student.objects.create(school=public_school)
@@ -589,26 +642,15 @@ class ManyToOneTests(TestCase):
         private_student = Student.objects.create(school=private_school)
 
         # Only one school is available via all() due to the custom default manager.
-        self.assertQuerysetEqual(School.objects.all(), ["<School: School object>"])
+        self.assertSequenceEqual(School.objects.all(), [public_school])
 
         self.assertEqual(public_student.school, public_school)
 
-        # Make sure the base manager is used so that an student can still access
+        # Make sure the base manager is used so that a student can still access
         # its related school even if the default manager doesn't normally
         # allow it.
         self.assertEqual(private_student.school, private_school)
 
-        # If the manager is marked "use_for_related_fields", it'll get used instead
-        # of the "bare" queryset. Usually you'd define this as a property on the class,
-        # but this approximates that in a way that's easier in tests.
-        School._default_manager.use_for_related_fields = True
-        try:
-            private_student = Student.objects.get(pk=private_student.pk)
-            with self.assertRaises(School.DoesNotExist):
-                private_student.school
-        finally:
-            School._default_manager.use_for_related_fields = False
-
         School._meta.base_manager_name = 'objects'
         School._meta._expire_cache()
         try:
@@ -624,3 +666,71 @@ class ManyToOneTests(TestCase):
         # doesn't exist should be an instance of a subclass of `AttributeError`
         # refs #21563
         self.assertFalse(hasattr(Article(), 'reporter'))
+
+    def test_clear_after_prefetch(self):
+        c = City.objects.create(name='Musical City')
+        District.objects.create(name='Ladida', city=c)
+        city = City.objects.prefetch_related('districts').get(id=c.id)
+        self.assertQuerysetEqual(city.districts.all(), ['<District: Ladida>'])
+        city.districts.clear()
+        self.assertQuerysetEqual(city.districts.all(), [])
+
+    def test_remove_after_prefetch(self):
+        c = City.objects.create(name='Musical City')
+        d = District.objects.create(name='Ladida', city=c)
+        city = City.objects.prefetch_related('districts').get(id=c.id)
+        self.assertQuerysetEqual(city.districts.all(), ['<District: Ladida>'])
+        city.districts.remove(d)
+        self.assertQuerysetEqual(city.districts.all(), [])
+
+    def test_add_after_prefetch(self):
+        c = City.objects.create(name='Musical City')
+        District.objects.create(name='Ladida', city=c)
+        d2 = District.objects.create(name='Ladidu')
+        city = City.objects.prefetch_related('districts').get(id=c.id)
+        self.assertEqual(city.districts.count(), 1)
+        city.districts.add(d2)
+        self.assertEqual(city.districts.count(), 2)
+
+    def test_set_after_prefetch(self):
+        c = City.objects.create(name='Musical City')
+        District.objects.create(name='Ladida', city=c)
+        d2 = District.objects.create(name='Ladidu')
+        city = City.objects.prefetch_related('districts').get(id=c.id)
+        self.assertEqual(city.districts.count(), 1)
+        city.districts.set([d2])
+        self.assertQuerysetEqual(city.districts.all(), ['<District: Ladidu>'])
+
+    def test_add_then_remove_after_prefetch(self):
+        c = City.objects.create(name='Musical City')
+        District.objects.create(name='Ladida', city=c)
+        d2 = District.objects.create(name='Ladidu')
+        city = City.objects.prefetch_related('districts').get(id=c.id)
+        self.assertEqual(city.districts.count(), 1)
+        city.districts.add(d2)
+        self.assertEqual(city.districts.count(), 2)
+        city.districts.remove(d2)
+        self.assertEqual(city.districts.count(), 1)
+
+    def test_cached_relation_invalidated_on_save(self):
+        """
+        Model.save() invalidates stale ForeignKey relations after a primary key
+        assignment.
+        """
+        self.assertEqual(self.a.reporter, self.r)  # caches a.reporter
+        self.a.reporter_id = self.r2.pk
+        self.a.save()
+        self.assertEqual(self.a.reporter, self.r2)
+
+    def test_cached_foreign_key_with_to_field_not_cleared_by_save(self):
+        parent = Parent.objects.create(name='a')
+        child = ToFieldChild.objects.create(parent=parent)
+        with self.assertNumQueries(0):
+            self.assertIs(child.parent, parent)
+
+    def test_reverse_foreign_key_instance_to_field_caching(self):
+        parent = Parent.objects.create(name='a')
+        ToFieldChild.objects.create(parent=parent)
+        child = parent.to_field_children.get()
+        with self.assertNumQueries(0):
+            self.assertIs(child.parent, parent)
