@@ -10,18 +10,25 @@ import sys
 import tempfile
 import warnings
 
-import django
-from django.apps import apps
-from django.conf import settings
-from django.db import connection, connections
-from django.test import TestCase, TransactionTestCase
-from django.test.runner import default_test_processes
-from django.test.selenium import SeleniumTestCaseBase
-from django.test.utils import get_runner
-from django.utils.deprecation import (
-    RemovedInDjango30Warning, RemovedInDjango31Warning,
-)
-from django.utils.log import DEFAULT_LOGGING
+try:
+    import django
+except ImportError as e:
+    raise RuntimeError(
+        'Django module not found, reference tests/README.rst for instructions.'
+    ) from e
+else:
+    from django.apps import apps
+    from django.conf import settings
+    from django.db import connection, connections
+    from django.test import TestCase, TransactionTestCase
+    from django.test.runner import default_test_processes
+    from django.test.selenium import SeleniumTestCaseBase
+    from django.test.utils import get_runner
+    from django.utils.deprecation import (
+        RemovedInDjango31Warning, RemovedInDjango40Warning,
+    )
+    from django.utils.log import DEFAULT_LOGGING
+    from django.utils.version import PY37
 
 try:
     import MySQLdb
@@ -32,7 +39,7 @@ else:
     warnings.filterwarnings('ignore', r'\(1003, *', category=MySQLdb.Warning)
 
 # Make deprecation warnings errors to ensure no usage of deprecated features.
-warnings.simplefilter("error", RemovedInDjango30Warning)
+warnings.simplefilter("error", RemovedInDjango40Warning)
 warnings.simplefilter('error', RemovedInDjango31Warning)
 # Make runtime warning errors to ensure no usage of error prone patterns.
 warnings.simplefilter("error", RuntimeWarning)
@@ -109,7 +116,7 @@ def get_installed():
     return [app_config.name for app_config in apps.get_app_configs()]
 
 
-def setup(verbosity, test_labels, parallel):
+def setup(verbosity, test_labels, parallel, start_at, start_after):
     # Reduce the given test labels to just the app module path.
     test_labels_set = set()
     for label in test_labels:
@@ -188,22 +195,33 @@ def setup(verbosity, test_labels, parallel):
         print('Aborting: A GIS database backend is required to run gis_tests.')
         sys.exit(1)
 
+    def _module_match_label(module_label, label):
+        # Exact or ancestor match.
+        return module_label == label or module_label.startswith(label + '.')
+
     # Load all the test model apps.
     test_modules = get_test_modules()
 
+    found_start = not (start_at or start_after)
     installed_app_names = set(get_installed())
     for modpath, module_name in test_modules:
         if modpath:
             module_label = modpath + '.' + module_name
         else:
             module_label = module_name
+        if not found_start:
+            if start_at and _module_match_label(module_label, start_at):
+                found_start = True
+            elif start_after and _module_match_label(module_label, start_after):
+                found_start = True
+                continue
+            else:
+                continue
         # if the module (or an ancestor) was named on the command line, or
         # no modules were named (i.e., run all), import
         # this module and add it to INSTALLED_APPS.
         module_found_in_labels = not test_labels or any(
-            # exact match or ancestor match
-            module_label == label or module_label.startswith(label + '.')
-            for label in test_labels_set
+            _module_match_label(module_label, label) for label in test_labels_set
         )
 
         if module_name in CONTRIB_TESTS_TO_APPS and module_found_in_labels:
@@ -234,7 +252,7 @@ def teardown(state):
     # Discard the multiprocessing.util finalizer that tries to remove a
     # temporary directory that's already removed by this script's
     # atexit.register(shutil.rmtree, TMPDIR) handler. Prevents
-    # FileNotFoundError at the end of a test run on Python 3.6+ (#27890).
+    # FileNotFoundError at the end of a test run (#27890).
     from multiprocessing.util import _finalizer_registry
     _finalizer_registry.pop((-100, 0), None)
 
@@ -265,8 +283,9 @@ class ActionSelenium(argparse.Action):
 
 
 def django_tests(verbosity, interactive, failfast, keepdb, reverse,
-                 test_labels, debug_sql, parallel, tags, exclude_tags):
-    state = setup(verbosity, test_labels, parallel)
+                 test_labels, debug_sql, parallel, tags, exclude_tags,
+                 test_name_patterns, start_at, start_after, pdb):
+    state = setup(verbosity, test_labels, parallel, start_at, start_after)
     extra_tests = []
 
     # Run the test suite, including the extra validation tests.
@@ -284,6 +303,8 @@ def django_tests(verbosity, interactive, failfast, keepdb, reverse,
         parallel=actual_test_processes(parallel),
         tags=tags,
         exclude_tags=exclude_tags,
+        test_name_patterns=test_name_patterns,
+        pdb=pdb,
     )
     failures = test_runner.run_tests(
         test_labels or get_installed(),
@@ -310,8 +331,8 @@ def get_subprocess_args(options):
     return subprocess_args
 
 
-def bisect_tests(bisection_label, options, test_labels, parallel):
-    state = setup(options.verbosity, test_labels, parallel)
+def bisect_tests(bisection_label, options, test_labels, parallel, start_at, start_after):
+    state = setup(options.verbosity, test_labels, parallel, start_at, start_after)
 
     test_labels = test_labels or get_installed()
 
@@ -334,22 +355,22 @@ def bisect_tests(bisection_label, options, test_labels, parallel):
         test_labels_b = test_labels[midpoint:] + [bisection_label]
         print('***** Pass %da: Running the first half of the test suite' % iteration)
         print('***** Test labels: %s' % ' '.join(test_labels_a))
-        failures_a = subprocess.call(subprocess_args + test_labels_a)
+        failures_a = subprocess.run(subprocess_args + test_labels_a)
 
         print('***** Pass %db: Running the second half of the test suite' % iteration)
         print('***** Test labels: %s' % ' '.join(test_labels_b))
         print('')
-        failures_b = subprocess.call(subprocess_args + test_labels_b)
+        failures_b = subprocess.run(subprocess_args + test_labels_b)
 
-        if failures_a and not failures_b:
+        if failures_a.returncode and not failures_b.returncode:
             print("***** Problem found in first half. Bisecting again...")
             iteration += 1
             test_labels = test_labels_a[:-1]
-        elif failures_b and not failures_a:
+        elif failures_b.returncode and not failures_a.returncode:
             print("***** Problem found in second half. Bisecting again...")
             iteration += 1
             test_labels = test_labels_b[:-1]
-        elif failures_a and failures_b:
+        elif failures_a.returncode and failures_b.returncode:
             print("***** Multiple sources of failure found")
             break
         else:
@@ -361,8 +382,8 @@ def bisect_tests(bisection_label, options, test_labels, parallel):
     teardown(state)
 
 
-def paired_tests(paired_test, options, test_labels, parallel):
-    state = setup(options.verbosity, test_labels, parallel)
+def paired_tests(paired_test, options, test_labels, parallel, start_at, start_after):
+    state = setup(options.verbosity, test_labels, parallel, start_at, start_after)
 
     test_labels = test_labels or get_installed()
 
@@ -410,7 +431,7 @@ if __name__ == "__main__":
         help='Tells Django to stop running the test suite after first failed test.',
     )
     parser.add_argument(
-        '-k', '--keepdb', action='store_true',
+        '--keepdb', action='store_true',
         help='Tells Django to preserve the test database between runs.',
     )
     parser.add_argument(
@@ -437,6 +458,10 @@ if __name__ == "__main__":
         '--selenium', action=ActionSelenium, metavar='BROWSERS',
         help='A comma-separated list of browsers to run the Selenium tests against.',
     )
+    parser.add_argument(
+        '--headless', action='store_true',
+        help='Run selenium tests in headless mode, if the browser supports the option.',
+    )
     parser.add_argument(
         '--selenium-hub',
         help='A URL for a selenium hub instance to use in combination with --selenium.',
@@ -463,6 +488,26 @@ if __name__ == "__main__":
         '--exclude-tag', dest='exclude_tags', action='append',
         help='Do not run tests with the specified tag. Can be used multiple times.',
     )
+    parser.add_argument(
+        '--start-after', dest='start_after',
+        help='Run tests starting after the specified top-level module.',
+    )
+    parser.add_argument(
+        '--start-at', dest='start_at',
+        help='Run tests starting at the specified top-level module.',
+    )
+    parser.add_argument(
+        '--pdb', action='store_true',
+        help='Runs the PDB debugger on error or failure.'
+    )
+    if PY37:
+        parser.add_argument(
+            '-k', dest='test_name_patterns', action='append',
+            help=(
+                'Only run test methods and classes matching test name pattern. '
+                'Same as unittest -k option. Can be used multiple times.'
+            ),
+        )
 
     options = parser.parse_args()
 
@@ -475,6 +520,18 @@ if __name__ == "__main__":
     # Allow including a trailing slash on app_labels for tab completion convenience
     options.modules = [os.path.normpath(labels) for labels in options.modules]
 
+    mutually_exclusive_options = [options.start_at, options.start_after, options.modules]
+    enabled_module_options = [bool(option) for option in mutually_exclusive_options].count(True)
+    if enabled_module_options > 1:
+        print('Aborting: --start-at, --start-after, and test labels are mutually exclusive.')
+        sys.exit(1)
+    for opt_name in ['start_at', 'start_after']:
+        opt_val = getattr(options, opt_name)
+        if opt_val:
+            if '.' in opt_val:
+                print('Aborting: --%s must be a top-level module.' % opt_name.replace('_', '-'))
+                sys.exit(1)
+            setattr(options, opt_name, os.path.normpath(opt_val))
     if options.settings:
         os.environ['DJANGO_SETTINGS_MODULE'] = options.settings
     else:
@@ -489,18 +546,27 @@ if __name__ == "__main__":
         if options.selenium_hub:
             SeleniumTestCaseBase.selenium_hub = options.selenium_hub
             SeleniumTestCaseBase.external_host = options.external_host
+        SeleniumTestCaseBase.headless = options.headless
         SeleniumTestCaseBase.browsers = options.selenium
 
     if options.bisect:
-        bisect_tests(options.bisect, options, options.modules, options.parallel)
+        bisect_tests(
+            options.bisect, options, options.modules, options.parallel,
+            options.start_at, options.start_after,
+        )
     elif options.pair:
-        paired_tests(options.pair, options, options.modules, options.parallel)
+        paired_tests(
+            options.pair, options, options.modules, options.parallel,
+            options.start_at, options.start_after,
+        )
     else:
         failures = django_tests(
             options.verbosity, options.interactive, options.failfast,
             options.keepdb, options.reverse, options.modules,
             options.debug_sql, options.parallel, options.tags,
             options.exclude_tags,
+            getattr(options, 'test_name_patterns', None),
+            options.start_at, options.start_after, options.pdb,
         )
         if failures:
             sys.exit(1)
