@@ -1,23 +1,20 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from datetime import datetime
+from functools import partialmethod
+from io import StringIO
+from unittest import mock, skipIf
 
 from django.core import serializers
 from django.core.serializers import SerializerDoesNotExist
 from django.core.serializers.base import ProgressBar
 from django.db import connection, transaction
 from django.http import HttpResponse
-from django.test import (
-    SimpleTestCase, mock, override_settings, skipUnlessDBFeature,
-)
+from django.test import SimpleTestCase, override_settings, skipUnlessDBFeature
 from django.test.utils import Approximate
-from django.utils.functional import curry
-from django.utils.six import StringIO
 
 from .models import (
-    Actor, Article, Author, AuthorProfile, BaseModel, Category, ComplexModel,
-    Movie, Player, ProxyBaseModel, ProxyProxyBaseModel, Score, Team,
+    Actor, Article, Author, AuthorProfile, BaseModel, Category, Child,
+    ComplexModel, Movie, Player, ProxyBaseModel, ProxyProxyBaseModel, Score,
+    Team,
 )
 
 
@@ -58,7 +55,7 @@ class SerializerRegistrationTests(SimpleTestCase):
             serializers.unregister_serializer("nonsense")
 
     def test_builtin_serializers(self):
-        "Requesting a list of serializer formats popuates the registry"
+        "Requesting a list of serializer formats populates the registry"
         all_formats = set(serializers.get_serializer_formats())
         public_formats = set(serializers.get_public_serializer_formats())
 
@@ -91,13 +88,9 @@ class SerializerRegistrationTests(SimpleTestCase):
             serializers.get_deserializer("nonsense")
 
 
-class SerializersTestBase(object):
+class SerializersTestBase:
     serializer_name = None  # Set by subclasses to the serialization format name
 
-    @staticmethod
-    def _comparison_value(value):
-        return value
-
     def setUp(self):
         sports = Category.objects.create(name="Sports")
         music = Category.objects.create(name="Music")
@@ -112,7 +105,7 @@ class SerializersTestBase(object):
             pub_date=datetime(2006, 6, 16, 11, 00)
         )
         self.a1.save()
-        self.a1.categories = [sports, op_ed]
+        self.a1.categories.set([sports, op_ed])
 
         self.a2 = Article(
             author=self.joe,
@@ -120,18 +113,16 @@ class SerializersTestBase(object):
             pub_date=datetime(2006, 6, 16, 13, 00, 11, 345)
         )
         self.a2.save()
-        self.a2.categories = [music, op_ed]
+        self.a2.categories.set([music, op_ed])
 
     def test_serialize(self):
-        """Tests that basic serialization works."""
-        serial_str = serializers.serialize(self.serializer_name,
-                                           Article.objects.all())
+        """Basic serialization works."""
+        serial_str = serializers.serialize(self.serializer_name, Article.objects.all())
         self.assertTrue(self._validate_output(serial_str))
 
     def test_serializer_roundtrip(self):
-        """Tests that serialized content can be deserialized."""
-        serial_str = serializers.serialize(self.serializer_name,
-                                           Article.objects.all())
+        """Serialized content can be deserialized."""
+        serial_str = serializers.serialize(self.serializer_name, Article.objects.all())
         models = list(serializers.deserialize(self.serializer_name, serial_str))
         self.assertEqual(len(models), 2)
 
@@ -146,11 +137,11 @@ class SerializersTestBase(object):
             # Serialize normally for a comparison
             string_data = serializers.serialize(self.serializer_name, [obj], indent=2)
 
-            # Check that the two are the same
+            # The two are the same
             if isinstance(stream, StringIO):
                 self.assertEqual(string_data, stream.getvalue())
             else:
-                self.assertEqual(string_data, stream.content.decode('utf-8'))
+                self.assertEqual(string_data, stream.content.decode())
 
     def test_serialize_specific_fields(self):
         obj = ComplexModel(field1='first', field2='second', field3='third')
@@ -162,20 +153,18 @@ class SerializersTestBase(object):
         )
         result = next(serializers.deserialize(self.serializer_name, serialized_data))
 
-        # Check that the deserialized object contains data in only the serialized fields.
+        # The deserialized object contains data in only the serialized fields.
         self.assertEqual(result.object.field1, 'first')
         self.assertEqual(result.object.field2, '')
         self.assertEqual(result.object.field3, 'third')
 
     def test_altering_serialized_output(self):
         """
-        Tests the ability to create new objects by
-        modifying serialized content.
+        The ability to create new objects by modifying serialized content.
         """
         old_headline = "Poker has no place on ESPN"
         new_headline = "Poker has no place on television"
-        serial_str = serializers.serialize(self.serializer_name,
-                                           Article.objects.all())
+        serial_str = serializers.serialize(self.serializer_name, Article.objects.all())
         serial_str = serial_str.replace(old_headline, new_headline)
         models = list(serializers.deserialize(self.serializer_name, serial_str))
 
@@ -192,27 +181,22 @@ class SerializersTestBase(object):
 
     def test_one_to_one_as_pk(self):
         """
-        Tests that if you use your own primary key field
-        (such as a OneToOneField), it doesn't appear in the
-        serialized field list - it replaces the pk identifier.
+        If you use your own primary key field (such as a OneToOneField), it
+        doesn't appear in the serialized field list - it replaces the pk
+        identifier.
         """
-        profile = AuthorProfile(author=self.joe,
-                                date_of_birth=datetime(1970, 1, 1))
-        profile.save()
-        serial_str = serializers.serialize(self.serializer_name,
-                                           AuthorProfile.objects.all())
+        AuthorProfile.objects.create(author=self.joe, date_of_birth=datetime(1970, 1, 1))
+        serial_str = serializers.serialize(self.serializer_name, AuthorProfile.objects.all())
         self.assertFalse(self._get_field_values(serial_str, 'author'))
 
         for obj in serializers.deserialize(self.serializer_name, serial_str):
-            self.assertEqual(obj.object.pk, self._comparison_value(self.joe.pk))
+            self.assertEqual(obj.object.pk, self.joe.pk)
 
     def test_serialize_field_subset(self):
-        """Tests that output can be restricted to a subset of fields"""
+        """Output can be restricted to a subset of fields"""
         valid_fields = ('headline', 'pub_date')
         invalid_fields = ("author", "categories")
-        serial_str = serializers.serialize(self.serializer_name,
-                                    Article.objects.all(),
-                                    fields=valid_fields)
+        serial_str = serializers.serialize(self.serializer_name, Article.objects.all(), fields=valid_fields)
         for field_name in invalid_fields:
             self.assertFalse(self._get_field_values(serial_str, field_name))
 
@@ -220,7 +204,7 @@ class SerializersTestBase(object):
             self.assertTrue(self._get_field_values(serial_str, field_name))
 
     def test_serialize_unicode(self):
-        """Tests that unicode makes the roundtrip intact"""
+        """Unicode makes the roundtrip intact"""
         actor_name = "Za\u017c\u00f3\u0142\u0107"
         movie_title = 'G\u0119\u015bl\u0105 ja\u017a\u0144'
         ac = Actor(name=actor_name)
@@ -261,7 +245,7 @@ class SerializersTestBase(object):
 
     def test_serialize_with_null_pk(self):
         """
-        Tests that serialized data with no primary key results
+        Serialized data with no primary key results
         in a model instance with no id
         """
         category = Category(name="Reference")
@@ -269,17 +253,15 @@ class SerializersTestBase(object):
         pk_value = self._get_pk_values(serial_str)[0]
         self.assertFalse(pk_value)
 
-        cat_obj = list(serializers.deserialize(self.serializer_name,
-                                               serial_str))[0].object
-        self.assertEqual(cat_obj.id, None)
+        cat_obj = list(serializers.deserialize(self.serializer_name, serial_str))[0].object
+        self.assertIsNone(cat_obj.id)
 
     def test_float_serialization(self):
-        """Tests that float values serialize and deserialize intact"""
+        """Float values serialize and deserialize intact"""
         sc = Score(score=3.4)
         sc.save()
         serial_str = serializers.serialize(self.serializer_name, [sc])
-        deserial_objs = list(serializers.deserialize(self.serializer_name,
-                                                serial_str))
+        deserial_objs = list(serializers.deserialize(self.serializer_name, serial_str))
         self.assertEqual(deserial_objs[0].object.score, Approximate(3.4, places=1))
 
     def test_deferred_field_serialization(self):
@@ -287,30 +269,26 @@ class SerializersTestBase(object):
         author = Author.objects.defer('name').get(pk=author.pk)
         serial_str = serializers.serialize(self.serializer_name, [author])
         deserial_objs = list(serializers.deserialize(self.serializer_name, serial_str))
-        # Check the class instead of using isinstance() because model instances
-        # with deferred fields (e.g. Author_Deferred_name) will pass isinstance.
-        self.assertEqual(deserial_objs[0].object.__class__, Author)
+        self.assertIsInstance(deserial_objs[0].object, Author)
 
     def test_custom_field_serialization(self):
-        """Tests that custom fields serialize and deserialize intact"""
+        """Custom fields serialize and deserialize intact"""
         team_str = "Spartak Moskva"
         player = Player()
         player.name = "Soslan Djanaev"
         player.rank = 1
         player.team = Team(team_str)
         player.save()
-        serial_str = serializers.serialize(self.serializer_name,
-                                           Player.objects.all())
+        serial_str = serializers.serialize(self.serializer_name, Player.objects.all())
         team = self._get_field_values(serial_str, "team")
         self.assertTrue(team)
         self.assertEqual(team[0], team_str)
 
         deserial_objs = list(serializers.deserialize(self.serializer_name, serial_str))
-        self.assertEqual(deserial_objs[0].object.team.to_string(),
-                         player.team.to_string())
+        self.assertEqual(deserial_objs[0].object.team.to_string(), player.team.to_string())
 
     def test_pre_1000ad_date(self):
-        """Tests that year values before 1000AD are properly formatted"""
+        """Year values before 1000AD are properly formatted"""
         # Regression for #12524 -- dates before 1000AD get prefixed
         # 0's on the year
         a = Article.objects.create(
@@ -324,11 +302,9 @@ class SerializersTestBase(object):
 
     def test_pkless_serialized_strings(self):
         """
-        Tests that serialized strings without PKs
-        can be turned into models
+        Serialized strings without PKs can be turned into models
         """
-        deserial_objs = list(serializers.deserialize(self.serializer_name,
-                                                     self.pkless_str))
+        deserial_objs = list(serializers.deserialize(self.serializer_name, self.pkless_str))
         for obj in deserial_objs:
             self.assertFalse(obj.object.id)
             obj.save()
@@ -346,7 +322,7 @@ class SerializersTestBase(object):
         })
 
     def test_deserialize_force_insert(self):
-        """Tests that deserialized content can be saved with force_insert as a parameter."""
+        """Deserialized content can be saved with force_insert as a parameter."""
         serial_str = serializers.serialize(self.serializer_name, [self.a1])
         deserial_obj = list(serializers.deserialize(self.serializer_name, serial_str))[0]
         with mock.patch('django.db.models.Model') as mock_model:
@@ -365,15 +341,46 @@ class SerializersTestBase(object):
         self.assertEqual(base_data, proxy_data.replace('proxy', ''))
         self.assertEqual(base_data, proxy_proxy_data.replace('proxy', ''))
 
+    def test_serialize_inherited_fields(self):
+        child_1 = Child.objects.create(parent_data='a', child_data='b')
+        child_2 = Child.objects.create(parent_data='c', child_data='d')
+        child_1.parent_m2m.add(child_2)
+        child_data = serializers.serialize(self.serializer_name, [child_1, child_2])
+        self.assertEqual(self._get_field_values(child_data, 'parent_m2m'), [])
+        self.assertEqual(self._get_field_values(child_data, 'parent_data'), [])
 
-class SerializersTransactionTestBase(object):
+
+class SerializerAPITests(SimpleTestCase):
+
+    def test_stream_class(self):
+        class File:
+            def __init__(self):
+                self.lines = []
+
+            def write(self, line):
+                self.lines.append(line)
+
+            def getvalue(self):
+                return ''.join(self.lines)
+
+        class Serializer(serializers.json.Serializer):
+            stream_class = File
+
+        serializer = Serializer()
+        data = serializer.serialize([Score(id=1, score=3.4)])
+        self.assertIs(serializer.stream_class, File)
+        self.assertIsInstance(serializer.stream, File)
+        self.assertEqual(data, '[{"model": "serializers.score", "pk": 1, "fields": {"score": 3.4}}]')
+
+
+class SerializersTransactionTestBase:
 
     available_apps = ['serializers']
 
     @skipUnlessDBFeature('supports_forward_references')
     def test_forward_refs(self):
         """
-        Tests that objects ids can be referenced before they are
+        Objects ids can be referenced before they are
         defined in the serialization data.
         """
         # The deserialization process needs to run in a transaction in order
@@ -391,16 +398,16 @@ class SerializersTransactionTestBase(object):
         self.assertEqual(art_obj.author.name, "Agnes")
 
 
-def register_tests(test_class, method_name, test_func, exclude=None):
+def register_tests(test_class, method_name, test_func, exclude=()):
     """
     Dynamically create serializer tests to ensure that all registered
     serializers are automatically tested.
     """
-    formats = [
-        f for f in serializers.get_serializer_formats()
-        if (not isinstance(serializers.get_serializer(f), serializers.BadSerializer)
-            and not f == 'geojson'
-            and (exclude is None or f not in exclude))
-    ]
-    for format_ in formats:
-        setattr(test_class, method_name % format_, curry(test_func, format_))
+    for format_ in serializers.get_serializer_formats():
+        if format_ == 'geojson' or format_ in exclude:
+            continue
+        decorated_func = skipIf(
+            isinstance(serializers.get_serializer(format_), serializers.BadSerializer),
+            'The Python library for the %s serializer is not installed.' % format_,
+        )(test_func)
+        setattr(test_class, method_name % format_, partialmethod(decorated_func, format_))
